#!/usr/local/bin/perl
# load gi_taxid_prot_table.pl
# Sheila Podell
# April 1, 2009

use warnings;
use strict;
use Getopt::Long;
use DBI;

# creates, populates a Mysql table of information on sequences in Genbank nr
# expects as input a darkhorse_config file and ncbi file gi_taxid_prot.dmp 
# 	ftp://ftp.ncbi.nih.gov/pub/taxonomy/gi_taxid_prot.dmp.gz

use warnings; 
use strict;
use Getopt::Long;
use DBI;

# get command line arguments, defaults 
	my ($db, $datafile, $sth, $sql, $overwrite, $update, $result, $config_filename);
	my $USAGE;
	my $message = qq(
	Usage: $0  -c config filename  )."\n\n";
	GetOptions( 
			    "c=s" => \$config_filename,
			     'o'   => \$overwrite,		#allow table overwrites
			    'u'   => \$update,
				);	     
	
	if($USAGE || !$config_filename) 
	{
		print STDERR $message;
		exit(0);
	} 
	
	unless (-s $config_filename)
	{
		print STDERR "  Couldn't find config file $config_filename \n";
		exit(0);
	}	

# get SQL database parameters from config file
    my %config_params= &read_config_file($config_filename);
	my $db_name = $config_params{db_name} || "not found";
	my $db_program = $config_params{db_program} || "not found";
	my $db_host = $config_params{db_host} || "not found";
	my $db_user = $config_params{db_user} || "not found";
	my $db_user_password = $config_params{db_user_password} || "not found";
	my $max_lines_per_packet = $config_params{max_lines_per_packet} || "not found";
	my $gi_taxid_prot_index = $config_params{gi_taxid_prot_dmp_path} || "not found";
	
	unless (-s "$gi_taxid_prot_index")
	{
		print STDERR "  Couldn't find gi_taxid_prot.dmp file $gi_taxid_prot_index \n";
		exit(0);
	}
	
	$datafile = $gi_taxid_prot_index;

# connect to database
	my $db_path = "DBI:$db_program:$db_name:$db_host:";
	my $dbh = DBI->connect($db_path, $db_user, $db_user_password) or die "Cannot connect: $DBI::errstr"; 	

# check to see if table already exists
	my $tablename = "gi_tax_prot";
	my $found_table = 0;
	$sql = "show tables;";
	$sth = $dbh->prepare($sql)
    	or dieStackTrace("Cannot prepare SELECT '$sql':\n<b>$DBI::errstr</b>");
	$sth->execute()
    	or dieStackTrace("Cannot execute SELECT '$sql':\n<b>$DBI::errstr</b>");
	my @row = ();
	my $element = ();
	while (@row = $sth->fetchrow_array)
	{
		foreach $element (@row)
		{
			if ($element =~/$tablename/)
			{
				$found_table = 1;
				unless (defined $overwrite || defined $update)
				{
					print STDERR "  Table $tablename already exists.\n";
					print STDERR "  Specify -o option if you wish to overwrite.\n";
					print STDERR "  Specify -u option to update without overwriting.\n";
					exit(0);
				}
				last;
			}
		}
	}
	
# if over-write option specified, drop table from database.
	if (defined $overwrite)
	{
		print STDERR "Over-writing table $tablename.\n";
		$sql = qq(DROP TABLE IF EXISTS `$tablename`);
		$result = $dbh->do($sql);		
	}

	if (defined $update)
	{
		print STDERR "updating table $tablename.\n";
	}

# Compose creation command
	if (defined $overwrite || $found_table == 0)
	{
		$sql = qq(CREATE TABLE `$tablename` (
	   `gi_num` int(11)  NOT NULL,
	   `ncbi_tax_id` int(11)  NULL,
		PRIMARY KEY  (`gi_num`),
		INDEX (ncbi_tax_id)
	 ) ;\n);
 		
 		$result = $dbh->do($sql); 	
		if ($result)
		{
			print STDERR "New table created.\n";
		}
		else
		{
			print STDERR "Unable to create new table";
			exit(0);
		}
	}
	
# compose database population command 
	my $gi_taxid_prot_index_wc = `wc -l $gi_taxid_prot_index`;
	$gi_taxid_prot_index_wc  =~ /(\d+).+/;
	my $total_num = $1;
	print STDERR "total num lines = $total_num\n";
	my $estimate = int($total_num/800000);
	print STDERR "  This table may take > $estimate minutes to load.\n";
	
	open (INPUT,"$datafile") or die "can't open input file $datafile\n $!\n";
	$sql  = " INSERT INTO $tablename VALUES ";
	print STDERR "  Populating table . . .\n";
	
	while (<INPUT>)
	{
		chomp;
		my @fields = split "\t", $_;
		unless (scalar @fields == 2)
		{
			print STDERR "\nSkipping problem line:\n";
			print STDERR "$_\n";
			next;
		}
		my $gi_num = "$fields[0]" || "NULL";
		my $ncbi_tax_id = "$fields[1]"  || "NULL";
		
			$sql .= "('$gi_num', '$ncbi_tax_id'),\n";
	# send insert statements in pieces to avoid exceeding MySql max_allowed_packet size	
		if ($. % $max_lines_per_packet == 0)
		{
			$sql = substr($sql, 0, -2); # get rid of terminal newline and comma
			$sql .= ";\n";
			$result = $dbh->do($sql);
			$sql  = " INSERT INTO $tablename VALUES ";
		# user feedback for long lists	
			if ($. % 100000 == 0)
			{
				print STDERR "  $. lines processed \n";
			}
		}
		elsif (eof)
		{
			print STDERR "  $. lines processed \n";
			$sql = substr($sql, 0, -2); # get rid of terminal newline and comma
			$sql .= ";\n";
			$result = $dbh->do($sql);
		}
	}	
	
	close INPUT;

# cleanup
	$sth->finish();
	$dbh->disconnect();

###########################
# SUBROUTINES
###########################

sub read_config_file
{
    my ($filename) = @_;
    open (INPUT, $filename)  or die "can't open DarkHorse config file $filename\n$!\n";
	my %dh_config = ();
	while (<INPUT>)
	{
		next if ($_ =~ /^\s+$/);
		next if ($_ =~ /^#/);
		if ($_ =~ /\[(.+)\]=\s*(.+)/)
		{
			$dh_config{$1}=$2;		
		}			
	}
	close INPUT;			
    return %dh_config;
}


__END__